/*
 *  The contents of this file are subject to the Netscape Public License
 *  Version 1.0 (the "License"); you may not use this file except in
 *  compliance with the License. You may obtain a copy of the License at
 *  http://www.mozilla.org/NPL/
 *
 *  Software distributed under the License is distributed on an "AS IS"
 *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *  License for the specific language governing rights and limitations under
 *  the License.
 *
 *  The Original Code is Mozilla Communicator client code, released March
 *  31, 1998.
 *
 *  The Initial Developer of the Original Code is Netscape Communications
 *  Corporation. Portions created by Netscape are Copyright (C) 1998 Netscape
 *  Communications Corporation. All Rights Reserved.
 *
 *  Contributors: Justin Fletcher (RISCOS Ltd) for use with FilerAction
 */

/*******************************************************************
 * File:     dboxlong.c
 * Purpose:  Support for longs in dbox
 * Author:   Justin Fletcher
 * Date:     24 May 1999
 ******************************************************************/

#include <string.h>
#include "dbox.h"
#include "options.h"
#include "prlong.h"

#ifdef USE_LONG_LONG

static void cvt_ll(PRInt64 num,char *buf)
{
    char cvtbuf[40];
    char *cvt;
    int negative=0;
    int digits;

    if (!LL_GE_ZERO(num)) {
      LL_NEG(num,num);
      negative=1;
    }

    /*
    ** Converting decimal is a little tricky. In the unsigned case we
    ** need to stop when we hit 10 digits. In the signed case, we can
    ** stop when the number is zero.
    */
    cvt = cvtbuf + sizeof(cvtbuf);
    digits = 0;
    while (!LL_IS_ZERO(num)) {
	PRInt32 digit;
	PRInt64 quot, rem;
	LL_UDIVMODL(&quot, &rem, num, 10);
	LL_L2I(digit, rem);
	*--cvt = digit + '0';
	digits++;
	num = quot;
    }
    if (digits == 0) {
	*--cvt = '0';
	digits++;
    }

    /*
    ** Now that we have the number converted without its sign, deal with
    ** the sign and zero padding.
    */
#ifdef USE_COMMAS
    if (digits>16)
    {
      memcpy(buf,cvt,digits);
      buf[digits]='\0';
    }
    else
    {
      int cnt;
      static char rem[21] = {
        0, 1, 2,
        0, 1, 2, /* 6 */
        0, 1, 2,
        0, 1, 2, /* 12 */
        0, 1, 2,
        0, 1, 2, /* 18 */
        0, 1, 2, /* 21 */
      };
      cnt=rem[digits-1];
      while (digits)
      {
        *buf++=*cvt++;
        digits--; cnt--;
        if (cnt<0)
          *buf++=',', cnt=2;
      }
      buf[-1]='\0';
    }
#else
    memcpy(buf,cvt,digits);
    buf[digits]='\0';
#endif
}

void dbox_setlongnumeric(dbox d, dbox_field f, PRInt64 n)
{
  char a[40];

  cvt_ll(n,a);
  /* strcpy(a,"unknown"); */

  dbox_setfield((dbox) d, f, a);
}

#endif
